/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.ui.handlers;

import java.util.Iterator;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import eu.chorevolution.studio.eclipse.core.ChorevolutionCorePlugin;
import eu.chorevolution.studio.eclipse.core.ChorevolutionCorePreferences;
import eu.chorevolution.studio.eclipse.core.internal.project.ChorevolutionProjectUtils;
import eu.chorevolution.studio.eclipse.core.internal.project.ChorevolutionSynthesisProjectNature;
import eu.chorevolution.studio.eclipse.ui.ChorevolutionUIMessages;
import eu.chorevolution.studio.eclipse.ui.ChorevolutionUIPlugin;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.ChoreographyDeploymentDescriptorGenerator;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.embedded.ChoreographyDeploymentDescriptorGeneratorEmbedded;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.rest.ChoreographyDeploymentDescriptorGeneratorRest;

public class GenerateChoreographyDeploymentDescriptor extends AbstractHandler {

	@Override
	public Object execute(ExecutionEvent event) {
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		IStructuredSelection selection = (IStructuredSelection) window.getSelectionService().getSelection();

		Iterator iterator = selection.iterator();
		boolean someErrorsOccurred = false;

		while (iterator.hasNext()) {
			IFile file = (IFile) iterator.next();
			if (ChorevolutionProjectUtils.isChorevolutionProject(file)) {
				try {
					IPath location = file.getLocation();
					if (location != null) {
						ChorevolutionCorePreferences prefs = ChorevolutionCorePreferences
								.getProjectOrWorkspacePreferences(file.getProject(), ChorevolutionCorePlugin.PLUGIN_ID);
						String isEmbeddedChorevolutionSynthesisProcessor = prefs.getString(
								ChorevolutionSynthesisProjectNature.PREF_SYNTHESIS_PROCESSOR_RUN_EMBEDDED, "true");
						ChoreographyDeploymentDescriptorGenerator choreographyDeploymentDescriptorGenerator = null;
						if (Boolean.valueOf(isEmbeddedChorevolutionSynthesisProcessor)) {
							choreographyDeploymentDescriptorGenerator = new ChoreographyDeploymentDescriptorGeneratorEmbedded(
									file.getProject(), file, new NullProgressMonitor());
						} else {
							choreographyDeploymentDescriptorGenerator = new ChoreographyDeploymentDescriptorGeneratorRest(
									file.getProject(), file, new NullProgressMonitor());
						}

						choreographyDeploymentDescriptorGenerator.storeChoreographyDeploymentDescriptor();

					}
				} catch (Exception e) {
					someErrorsOccurred = true;
					MessageDialog.openError(ChorevolutionUIPlugin.getActiveWorkbenchShell(),
							ChorevolutionUIMessages.TransformatorORGeneator_Error,
							NLS.bind(ChorevolutionUIMessages.Transformator_chorarch2choreospecError, file.getName(),
									e.getLocalizedMessage()));
				}
			}

		}

		if (!someErrorsOccurred) {
			MessageDialog.openInformation(ChorevolutionUIPlugin.getActiveWorkbenchShell(),
					ChorevolutionUIMessages.TransformatorORGeneator_Information,
					ChorevolutionUIMessages.Transformator_chorarch2choreospecInformation);
		}

		return null;
	}

}
