/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.embedded;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import eu.chorevolution.studio.eclipse.core.ChorevolutionCoreUtils;
import eu.chorevolution.studio.eclipse.core.internal.project.model.impl.ChorevolutionSynthesisProjectStructureFolder;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionSynthesisSourceModelPrefs;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.ChoreographyArchitectureGenerator;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.GeneratedArtifactSynthesisProcessor;
import eu.chorevolution.transformations.generativeapproach.choreographyarchitecturegenerator.ChoreographyArchitectureGeneratorRequest;
import eu.chorevolution.transformations.generativeapproach.choreographyarchitecturegenerator.ChoreographyArchitectureGeneratorResponse;

public class ChoreographyArchitectureGeneratorEmbedded extends ChoreographyArchitectureGenerator{

	
	private GeneratedArtifactSynthesisProcessor generatedArtifactSynthesisProcessor;
	private IFile bpmnFile;
	private IProgressMonitor monitor;
	
	public ChoreographyArchitectureGeneratorEmbedded(
			GeneratedArtifactSynthesisProcessor generatedArtifactSynthesisProcessor, IFile bpmnFile, IProject project,
			IProgressMonitor monitor) {
		super(project);
		this.generatedArtifactSynthesisProcessor = generatedArtifactSynthesisProcessor;
		this.bpmnFile = bpmnFile;
		this.monitor = monitor;
	}

	@Override
	public void storeChoreographyArchitecture()
			throws Exception {
		monitor.beginTask("Choreography Architecture Generator", 1);
		
		ChoreographyArchitectureGeneratorRequest bpmn2ChoreoArchTransformatorRequest = new ChoreographyArchitectureGeneratorRequest(
				FileUtils.readFileToByteArray(bpmnFile.getRawLocation().makeAbsolute().toFile()),
				generatedArtifactSynthesisProcessor.getClientComponents(),
				generatedArtifactSynthesisProcessor.getProsumerComponents(), 
				generatedArtifactSynthesisProcessor.getProviderComponents());

		ChoreographyArchitectureGeneratorResponse choreographyArchitectureGeneratorResponse = new eu.chorevolution.transformations.generativeapproach.choreographyarchitecturegenerator.ChoreographyArchitectureGenerator().generate(bpmn2ChoreoArchTransformatorRequest);

		IFolder architectureFolder = bpmnFile.getProject()
				.getFolder(ChorevolutionSynthesisProjectStructureFolder.SYNTHESIS_PROCESSOR_FOLDER_NAME)
				.getFolder(super.getPropertyValues()
						.get(ChorevolutionSynthesisSourceModelPrefs.PREF_SYNTHESISPROCESSOR_ARCHITECTURALSTYLE)
						.getValue());

		ChorevolutionCoreUtils.createFile(ChorevolutionCoreUtils.changeExtension(bpmnFile.getName(), CHOREOGRAPHY_ARCHITECTURE_FILE_EXTENSION),
				architectureFolder, choreographyArchitectureGeneratorResponse.getChoreographyArchitecture());

		super.getProject().refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());

		monitor.worked(1);
	}

}
