/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.embedded;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import eu.chorevolution.studio.eclipse.core.ChorevolutionCoreUtils;
import eu.chorevolution.studio.eclipse.core.internal.project.model.impl.ChorevolutionSynthesisProjectStructureFolder;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionSynthesisSourceModelPrefs;
import eu.chorevolution.studio.eclipse.ui.ChorevolutionUIPlugin;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.CoordinationDelegateGeneration;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.GeneratedArtifactSynthesisProcessor;
import eu.chorevolution.studio.eclipse.ui.handlers.wizard.model.CorrelationChoreographyTaskTableViewerRecord;
import eu.chorevolution.studio.eclipse.ui.handlers.wizard.model.ParticipantTableViewerRecord;
import eu.chorevolution.transformations.generativeapproach.cdconsumerpartgenerator.CDConsumerPartGenerator;
import eu.chorevolution.transformations.generativeapproach.cdconsumerpartgenerator.impl.CDConsumerPartGeneratorImpl;
import eu.chorevolution.transformations.generativeapproach.cdgenerator.CDGenerator;
import eu.chorevolution.transformations.generativeapproach.cdgenerator.impl.CDGeneratorImpl;
import eu.chorevolution.transformations.generativeapproach.cdgenerator.model.CD;
import eu.chorevolution.transformations.generativeapproach.cdgenerator.model.ProjectionData;
import eu.chorevolution.transformations.generativeapproach.cdgenerator.model.TaskCorrelationData;
import eu.chorevolution.transformations.generativeapproach.cdgenerator.model.WSDLData;
import eu.chorevolution.transformations.generativeapproach.choreographyarchitecturegenerator.model.ComponentData;

public class CoordinationDelegateGenerationEmbedded extends CoordinationDelegateGeneration{

	private GeneratedArtifactSynthesisProcessor generatedArtifactSynthesisProcessor;
	private IProgressMonitor monitor;
	private List<WSDLData> wsdlDatas;
	private Map<String, List<CorrelationChoreographyTaskTableViewerRecord>> correlationChoreographyTasks;
	
	public CoordinationDelegateGenerationEmbedded(List<ParticipantTableViewerRecord> prosumerParticipants,
			List<ParticipantTableViewerRecord> clientParticipants, List<WSDLData> wsdlDatas,
			Map<String, List<CorrelationChoreographyTaskTableViewerRecord>> correlationChoreographyTasks,
			GeneratedArtifactSynthesisProcessor generatedArtifactSynthesisProcessor, IProject project,
			IProgressMonitor monitor) {
	
		super(project, prosumerParticipants, clientParticipants);

		this.generatedArtifactSynthesisProcessor = generatedArtifactSynthesisProcessor;
		this.monitor = monitor;
		this.wsdlDatas = wsdlDatas;
		this.correlationChoreographyTasks = correlationChoreographyTasks;

	}

	
	@Override
	public void storeCoordinatioDelegates() throws Exception {
		monitor.beginTask("Coordination Delegate Generator", super.getProsumerParticipants().size() + super.getClientParticipants().size());

		for (ParticipantTableViewerRecord participantTableViewerRecord : super.getProsumerParticipants()) {
			if (monitor.isCanceled()) {
				return;
			}

			monitor.subTask("Generate the Coordination Delegate for Prosumer Participant: "
					+ participantTableViewerRecord.getRecordLabel());
			storeProsumerCoordinationDelegate(participantTableViewerRecord);
			monitor.worked(1);
		}

		for (ParticipantTableViewerRecord participantTableViewerRecord : super.getClientParticipants()) {
			if (monitor.isCanceled()) {
				return;
			}

			monitor.subTask("Generate the Coordination Delegate for Client Participant: "
					+ participantTableViewerRecord.getRecordLabel());

			storeClientCoordinationDelegate(participantTableViewerRecord);
			monitor.worked(1);
		}

	}

	private void storeProsumerCoordinationDelegate(ParticipantTableViewerRecord participantTableViewerRecord)
			throws Exception {
		IFolder participantFolder = super.getProject()
				.getFolder(ChorevolutionSynthesisProjectStructureFolder.SYNTHESIS_PROCESSOR_FOLDER_NAME)
				.getFolder(super.getPropertyValues()
						.get(ChorevolutionSynthesisSourceModelPrefs.PREF_SYNTHESISPROCESSOR_COORD).getValue())
				.getFolder(participantTableViewerRecord.getParticipant());

		IFolder artifactFolder = ChorevolutionCoreUtils.createFolder(ChorevolutionSynthesisProjectStructureFolder.ARTIFACTS_FOLDER_NAME, participantFolder);
		IFolder modelFolder = participantFolder
				.getFolder(ChorevolutionSynthesisProjectStructureFolder.MODELS_FOLDER_NAME);

		IFile projectedFile = ChorevolutionUIPlugin.getWorkspace().getRoot().getFile(modelFolder.getFullPath()
				.append(participantTableViewerRecord.getServiceProjectName()).addFileExtension(BPMN2_FILE_EXTENSION));
		IFile xsdTypesFile = ChorevolutionUIPlugin.getWorkspace().getRoot()
				.getFile(modelFolder.getFullPath().append(XSD_FILE_NAME).addFileExtension(XSD_FILE_EXTENSION));

		ProjectionData projectionData = new ProjectionData();
		projectionData.setParticipantName(participantTableViewerRecord.getParticipant());
		projectionData.setCdName(participantTableViewerRecord.getServiceProjectName());
		projectionData.setBpmn(FileUtils.readFileToByteArray(projectedFile.getRawLocation().makeAbsolute().toFile()));
		projectionData.setTypes(FileUtils.readFileToByteArray(xsdTypesFile.getRawLocation().makeAbsolute().toFile()));

		List<String> clientParticipantsName = new ArrayList<String>();
		super.getClientParticipants().forEach(item -> {
			clientParticipantsName.add(item.getParticipant());
		});
		
		CDGenerator cdGenerator = new CDGeneratorImpl();
		CD cd = cdGenerator.generateCDProsumer(super.getAllProsumerAndClientParticipantName(), clientParticipantsName, projectionData, wsdlDatas);

		IFile coordinationDelegate = ChorevolutionCoreUtils.createFile(cd.getName() + "." + TARGZ_EXTENSION, artifactFolder, cd.getArtifact());
		ComponentData prosumerComponentData = generatedArtifactSynthesisProcessor.addProsumerComponent(participantTableViewerRecord.getParticipant(), 
				participantTableViewerRecord.getServiceProjectName(), EMBEDDED_PREFIX_LCOATION+coordinationDelegate.getRawLocation().makeAbsolute().toOSString(),true);
		
		
		
		// create Coordination Delegate Consumer part
		CDConsumerPartGenerator cdConsumerPartGenerator = new CDConsumerPartGeneratorImpl();
		byte[] cdConsumerPartContent = cdConsumerPartGenerator
				.generateCDConsumerPart(participantTableViewerRecord.getServiceProjectName().replaceFirst("cd", ""), cd.getProsumerwsdl());

		String projectName = ChorevolutionCoreUtils.unTar(cdConsumerPartContent,
				ChorevolutionUIPlugin.getWorkspace().getRoot().getLocation());
		IProject importedProject = ChorevolutionCoreUtils.importMavenProject(ChorevolutionUIPlugin.getWorkspace().getRoot().getLocation().toFile(),
				projectName);
		
		generatedArtifactSynthesisProcessor.addConsumerComponentToProsumerComponent(
				prosumerComponentData,
				participantTableViewerRecord.getServiceProjectName().replaceFirst("cd", ""),
				EMBEDDED_PREFIX_LCOATION+ChorevolutionUIPlugin.getWorkspace().getRoot().getLocation().append(projectName).append("target").append(projectName).addFileExtension(WAR_EXTENSION).makeAbsolute().toOSString());
		super.getProject().refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());
	}

	private void storeClientCoordinationDelegate(ParticipantTableViewerRecord participantTableViewerRecord)
			throws Exception {
		IFolder participantFolder = super.getProject()
				.getFolder(ChorevolutionSynthesisProjectStructureFolder.SYNTHESIS_PROCESSOR_FOLDER_NAME)
				.getFolder(super.getPropertyValues()
						.get(ChorevolutionSynthesisSourceModelPrefs.PREF_SYNTHESISPROCESSOR_COORD).getValue())
				.getFolder(participantTableViewerRecord.getParticipant());

		IFolder artifactFolder = ChorevolutionCoreUtils
				.createFolder(ChorevolutionSynthesisProjectStructureFolder.ARTIFACTS_FOLDER_NAME, participantFolder);
		IFolder modelFolder = participantFolder
				.getFolder(ChorevolutionSynthesisProjectStructureFolder.MODELS_FOLDER_NAME);

		IFile projectedFile = ChorevolutionUIPlugin.getWorkspace().getRoot().getFile(modelFolder.getFullPath()
				.append(participantTableViewerRecord.getServiceProjectName()).addFileExtension(BPMN2_FILE_EXTENSION));
		IFile xsdTypesFile = ChorevolutionUIPlugin.getWorkspace().getRoot()
				.getFile(modelFolder.getFullPath().append(XSD_FILE_NAME).addFileExtension(XSD_FILE_EXTENSION));

		ProjectionData projectionData = new ProjectionData();
		projectionData.setParticipantName(participantTableViewerRecord.getParticipant());
		projectionData.setCdName(participantTableViewerRecord.getServiceProjectName());
		projectionData.setBpmn(FileUtils.readFileToByteArray(projectedFile.getRawLocation().makeAbsolute().toFile()));
		projectionData.setTypes(FileUtils.readFileToByteArray(xsdTypesFile.getRawLocation().makeAbsolute().toFile()));

		List<TaskCorrelationData> correlationDatas = new ArrayList<TaskCorrelationData>();
		for (CorrelationChoreographyTaskTableViewerRecord correlationChoreographyTask : correlationChoreographyTasks
				.get(participantTableViewerRecord.getParticipant())) {
			if (!correlationChoreographyTask.getCorrelatedWith()
					.equals(CorrelationChoreographyTaskTableViewerRecord.NONE_CORRELLATION)) {
				correlationDatas.add(new TaskCorrelationData(correlationChoreographyTask.getChoreographyTask(),
						correlationChoreographyTask.getCorrelatedWith()));
			}
		}

		CDGenerator cdGenerator = new CDGeneratorImpl();
		CD cd = cdGenerator.generateCDClient(correlationDatas, projectionData, wsdlDatas);

		IFile clientComponent = ChorevolutionCoreUtils.createFile(cd.getName() + "." + TARGZ_EXTENSION, artifactFolder, cd.getArtifact());
		generatedArtifactSynthesisProcessor.addClientComponent(participantTableViewerRecord.getParticipant(), 
				participantTableViewerRecord.getServiceProjectName(), EMBEDDED_PREFIX_LCOATION+clientComponent.getRawLocation().makeAbsolute().toOSString(),true);
		
		super.getProject().refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());
	}

}
