/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.rest;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import eu.chorevolution.studio.eclipse.core.ChorevolutionCoreUtils;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionServicesURIPrefs;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionSynthesisSourceModelPrefs;
import eu.chorevolution.studio.eclipse.ui.handlers.synthesisprocessor.ChoreographyDeploymentDescriptorGenerator;
import eu.chorevolution.synthesisprocessor.rest.api.ChoreographyDeploymentDescriptorGeneratorRequest;
import eu.chorevolution.synthesisprocessor.rest.api.ChoreographyDeploymentDescriptorGeneratorResponse;
import eu.chorevolution.synthesisprocessor.rest.api.client.ChoreographyDeploymentDescriptorGeneratorClient;

public class ChoreographyDeploymentDescriptorGeneratorRest extends ChoreographyDeploymentDescriptorGenerator{
	
	private IFile choreographyArchitecture;
	private IProgressMonitor monitor;
	private ChoreographyDeploymentDescriptorGeneratorClient client;
	

	public ChoreographyDeploymentDescriptorGeneratorRest(IProject project, IFile choreographyArchitecture, IProgressMonitor monitor) {
		super(project);
		this.choreographyArchitecture = choreographyArchitecture;
		this.monitor = monitor;
		this.client = new ChoreographyDeploymentDescriptorGeneratorClient(super.getPropertyValues().get(ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_CHOREOGRAPHY_DEPLOYMENT_DESCRIPTION_GENERATOR_URI).getValue());
	}

	@Override
	public void storeChoreographyDeploymentDescriptor()
			throws Exception {
		monitor.beginTask("Choreography Deployment Descriptor Generator", 1);
		
		
		IFolder choreographySpecificationFolder = super.getProject().getFolder(
				super.getPropertyValues().get(ChorevolutionSynthesisSourceModelPrefs.PREF_CHOREOGRAPHYDEPLOYMENT).getValue());

		ChoreographyDeploymentDescriptorGeneratorRequest choreographySpecificationGeneratorRequest = new ChoreographyDeploymentDescriptorGeneratorRequest(
				FileUtils.readFileToByteArray(choreographyArchitecture.getRawLocation().makeAbsolute().toFile()));

		ChoreographyDeploymentDescriptorGeneratorResponse choreographySpecificationGeneratorResponse = client.generateChoreographyDeploymentDescriptor(choreographySpecificationGeneratorRequest);
		
		ChorevolutionCoreUtils.createFile(ChorevolutionCoreUtils.changeExtension(choreographyArchitecture.getName(), CHOREOGRAPHY_DEPLOYMENT_DESCRIPTION_FILE_EXTENSION),
				choreographySpecificationFolder,
				choreographySpecificationGeneratorResponse.getChoreographyDeploymentDescriptorContent());
		choreographySpecificationFolder.refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());

		super.getProject().refreshLocal(IResource.DEPTH_ZERO, new NullProgressMonitor());

		monitor.worked(1);
	}

}
