/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.wizard.template.chorevolutionsynthesis;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;

import eu.chorevolution.studio.eclipse.core.ChorevolutionCorePlugin;
import eu.chorevolution.studio.eclipse.core.ChorevolutionCorePreferences;
import eu.chorevolution.studio.eclipse.core.internal.project.ChorevolutionProjectUtils;
import eu.chorevolution.studio.eclipse.core.internal.project.ChorevolutionSynthesisProjectNature;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionPreferenceData;
import eu.chorevolution.studio.eclipse.wizard.template.Utilities;


public class ChorevolutionSynthesisProjectWizard extends Wizard implements INewWizard, IExecutableExtension  {
	private static final String WIZARD_NAME = ChorevolutionSynthesisProjectWizardMessages.NewProject_windowTitle;

	private ChorevolutionSynthesisProjectWizardMainPage mainPage;
	private ChorevolutionSynthesisProjectWizardStructurePage structurePage;
	private ISelection selection;
	private IConfigurationElement configurationElement;

	/**
	 * Constructor for ChorevolutionProjectWizard.
	 */
	public ChorevolutionSynthesisProjectWizard() {
		setWindowTitle(WIZARD_NAME);
		setNeedsProgressMonitor(true);
	}

	/**
	 * We will accept the selection in the workbench to see if we can initialize from it.
	 * 
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}

	/**
	 * Adding the pages to the wizard.
	 */
	@Override
	public void addPages() {
		mainPage = new ChorevolutionSynthesisProjectWizardMainPage(selection);
		addPage(mainPage);

		structurePage = new ChorevolutionSynthesisProjectWizardStructurePage(selection);
		addPage(structurePage);
	}

	@Override
	public void createPageControls(Composite pageContainer) {
		super.createPageControls(pageContainer);
		// TODO help
		// PlatformUI.getWorkbench().getHelpSystem().setHelp(getShell(), IHelpContexts.New_File_Wizard);
	}

	@Override
	public void setInitializationData(IConfigurationElement config, String propertyName, Object data) throws CoreException {
		configurationElement = config;
	}
	
	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We will create an operation and run it using wizard as execution context.
	 */
	@Override
	public boolean performFinish() {
		String projectName = this.mainPage.getProjectName();
		URI location = null;
		if (!this.mainPage.useDefaults()) {
			location = mainPage.getLocationURI();
		}
		List<ChorevolutionPreferenceData> preferences = new ArrayList<ChorevolutionPreferenceData>();
		preferences.addAll(mainPage.getSynthesisProcessorURIs());
		preferences.addAll(structurePage.getProjectSettings());
		
		ChorevolutionProjectUtils.createProjectWithNature(projectName, ChorevolutionCorePlugin.SYNTHESIS_NATURE_ID, preferences, location, structurePage.getStructureFolders(), new NullProgressMonitor());

		//create property to set the embedded of rest use of the synthesis processor and configure BPMN2 nature to the created project
		for(IProject project :ChorevolutionProjectUtils.getChorevolutionProjects()){
		    if(project.getName().equals(projectName)){
		    	// create property to set the embedded of rest use of the synthesis processor
		    	ChorevolutionCorePreferences prefs = ChorevolutionCorePreferences.getProjectOrWorkspacePreferences(project, ChorevolutionCorePlugin.PLUGIN_ID);
		        prefs.putString(ChorevolutionSynthesisProjectNature.PREF_SYNTHESIS_PROCESSOR_RUN_EMBEDDED, String.valueOf(mainPage.isSynthesisProcessorEmbeddedMode()));
		        
				// configure BPMN2 project nature
				Utilities.configureBPMN2Nature(project, true);
		    }
		}
		
		
		
		// now the wizard automatically opens the correct perspective
		BasicNewProjectResourceWizard.updatePerspective(configurationElement);

		return true;
	}
	
	
	

}