/*
  * Copyright 2015 The CHOReVOLUTION project
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
package eu.chorevolution.studio.eclipse.wizard.template.chorevolutionsynthesis;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;

import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionPreferenceData;
import eu.chorevolution.studio.eclipse.core.preferences.ChorevolutionServicesURIPrefs;
import eu.chorevolution.studio.eclipse.wizard.template.Utilities;

public class ChorevolutionSynthesisProjectWizardMainPage extends WizardNewProjectCreationPage {

	private static final String PAGE_NAME = ChorevolutionSynthesisProjectWizardMessages.NewProject_mainPageName;

	private ISelection selection;
	private Button embeddedSynthesisProcessorButton;
	private Button restSynthesisProcessorButton;
	private Map<String, Text> textWidgets;
	private Button resetButton;
	
	public ChorevolutionSynthesisProjectWizardMainPage(ISelection selection) {
		super(PAGE_NAME);
		this.selection = selection;
		setTitle(ChorevolutionSynthesisProjectWizardMessages.NewProject_projectTitle);
		setDescription(ChorevolutionSynthesisProjectWizardMessages.NewProject_projectDescription);
		textWidgets = new HashMap<String, Text>();
	}

	@Override
	public void createControl(Composite parent) {
		super.createControl(parent);

		Control control = this.getControl();
		if (control instanceof Composite) {

			Composite composite = (Composite) control;
			GridLayout layout = new GridLayout();
			composite.setLayout(layout);

			Group group = new Group(composite, SWT.SHADOW_ETCHED_IN);
			group.setText("Synthesis Processor Configuration");
			group.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

			GridLayout gridLayout = new GridLayout(2, false);
			gridLayout.marginTop = 10;
			gridLayout.marginBottom = 10;
			group.setLayout(gridLayout);
			embeddedSynthesisProcessorButton = new Button(group, SWT.RADIO);
			embeddedSynthesisProcessorButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1));
			embeddedSynthesisProcessorButton.setText("Internal");
			embeddedSynthesisProcessorButton.setSelection(false);
			embeddedSynthesisProcessorButton.addSelectionListener(selectSynthesisProcessorConfiguration);

			restSynthesisProcessorButton = new Button(group, SWT.RADIO);
			restSynthesisProcessorButton.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1));
			restSynthesisProcessorButton.setText("External (REST)");
			restSynthesisProcessorButton.setSelection(true);
			restSynthesisProcessorButton.addSelectionListener(selectSynthesisProcessorConfiguration);
			
			Map<String, ChorevolutionPreferenceData> projectOrWorkspacePreferencesValues = (new ChorevolutionServicesURIPrefs())
					.readProjectOrWorkspacePreferences(null);
			List<ChorevolutionPreferenceData> synthesisProcessorRestURI = new ArrayList<ChorevolutionPreferenceData>();
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues
					.get(ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_BPMN2_CHOREOGRAPHY_PROJECTOR_URI));
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues
					.get(ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_BPMN2_CHOREOGRAPHY_VALIDATOR_URI));
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues
					.get(ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_CD_GENERATOR_URI));
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues
					.get(ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_BC_GENERATOR_URI));
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues
					.get(ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_SF_GENERATOR_URI));
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues.get(
					ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_CHOREOGRAPHY_ARCHITECTURE_GENERATOR_URI));
			synthesisProcessorRestURI.add(projectOrWorkspacePreferencesValues.get(
					ChorevolutionServicesURIPrefs.PREF_SYNTHESIS_PROCESSOR_CHOREOGRAPHY_DEPLOYMENT_DESCRIPTION_GENERATOR_URI));

			createListenerTexts(Utilities.createGrupContents(group,"",
					synthesisProcessorRestURI, false));
			
			resetButton = new Button(group, SWT.PUSH);
			resetButton.setText("Restore Defaults");
			resetButton.setLayoutData(new GridData(SWT.RIGHT, SWT.BOTTOM, true, true,2,1));
			resetButton.addSelectionListener(resetListener);
			
			enableOrDisableSynthesisProcessorURIs(true);
		}
	}

	private SelectionListener selectSynthesisProcessorConfiguration = new SelectionAdapter() {
		@Override
		public void widgetSelected(SelectionEvent e) {
			if ((Button) e.widget == embeddedSynthesisProcessorButton) {
				enableOrDisableSynthesisProcessorURIs(false);
				setErrorMessage(null);
			} else if ((Button) e.widget == restSynthesisProcessorButton) {
				enableOrDisableSynthesisProcessorURIs(true);
			}
		}
	};

	private void enableOrDisableSynthesisProcessorURIs(boolean enabled) {
		for (Text textURI : textWidgets.values()) {
			textURI.setEnabled(enabled);
		}
		resetButton.setEnabled(enabled);
	}

	private void createListenerTexts(Map<String, Text> texts) {
		for (Map.Entry<String, Text> entry : texts.entrySet()) {
			createListenerText(entry.getKey(), entry.getValue());
		}
	}

	private void createListenerText(String key, Text text) {
		text.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				if (validateWizardPage()) {
					setPageComplete(true);
					setErrorMessage(null);
				} else {
					setPageComplete(false);
					setErrorMessage(ChorevolutionSynthesisProjectWizardMessages.NewProject_synthesisProcessorURIError);
				}
			}
		});
		textWidgets.put(key, text);
	}

	private boolean validateWizardPage() {
		for (String value : textWidgets.keySet()) {
			if (textWidgets.get(value).getText().trim().isEmpty())
				return false;
		}
		return true;
	}
	
	private SelectionListener resetListener = new SelectionAdapter() {
		@Override
		public void widgetSelected(SelectionEvent e) {
			if ((Button) e.widget == resetButton) {
				performDefaults();
			}
		}
	};
	
	private void performDefaults() {
		// read workspace preferences
		Map<String, ChorevolutionPreferenceData> propertyValues = (new ChorevolutionServicesURIPrefs()).readProjectOrWorkspacePreferences(null);
		for (Map.Entry<String, ChorevolutionPreferenceData> entry : propertyValues.entrySet()) {
			if (textWidgets.get(entry.getKey()) != null){
				textWidgets.get(entry.getKey()).setText(entry.getValue().getValue());
				textWidgets.get(entry.getKey()).update();
			}
		}

	}
	
	public List<ChorevolutionPreferenceData>  getSynthesisProcessorURIs() {
		List<ChorevolutionPreferenceData> projectSettings = new ArrayList<ChorevolutionPreferenceData>();
		// get preferences value for workspace or selected project if specified
		Map<String, ChorevolutionPreferenceData> projectOrWorkspacePreferencesValues = (new ChorevolutionServicesURIPrefs()).readProjectOrWorkspacePreferences(null);

		for (Map.Entry<String, Text> entry : textWidgets.entrySet()) {
			ChorevolutionPreferenceData chorevolutionPreferenceData = new ChorevolutionPreferenceData(
					projectOrWorkspacePreferencesValues.get(entry.getKey()).getID(),
					projectOrWorkspacePreferencesValues.get(entry.getKey()).getLabel(),
					entry.getValue().getText(),
					projectOrWorkspacePreferencesValues.get(entry.getKey()).getDescriptionProperty());
			projectSettings.add(chorevolutionPreferenceData);
		}
		return projectSettings;

	}

	public boolean isSynthesisProcessorEmbeddedMode() {
		if (embeddedSynthesisProcessorButton.getSelection()) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public boolean canFlipToNextPage() {
		return super.validatePage();
	}

}
